'use strict';

exports.__esModule = true;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _progress = require('progress');

var _progress2 = _interopRequireDefault(_progress);

var _child_process = require('child_process');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var pkg = require(_path2.default.join(__dirname, '../package.json'));
var isTTY = process.stderr.isTTY && !process.env.CI;

_commander2.default.option('-b, --beta', 'Check for beta releases').option('-y, --force-yes', 'Update without user confirmation').parse(process.argv);

var RELEASES_API = 'https://api.github.com/repos/atom/atom/releases';
var DOWNLOAD_URL = 'https://github.com/atom/atom/releases/download';
var RELEASE_URL = 'https://github.com/atom/atom/releases/tag';
var USER_AGENT = `atom-check-updates/${pkg.version}`;
var TEMP_FOLDER = '/tmp/acu';
var DISTRO_DEBIAN = Symbol('deb');
var DISTRO_RPM = Symbol('rpm');

var distroConfig = {
  [DISTRO_DEBIAN]: {
    name: 'Debian',
    file: 'atom-amd64.deb',
    cmd: ['dpkg', '-i']
  },
  [DISTRO_RPM]: {
    name: 'RPM',
    file: 'atom.x86_64.rpm',
    cmd: ['rpm', '-Uvh']
  }
};

var bin = _commander2.default.beta ? 'atom-beta' : 'atom';

var getDebianConfig = function getDebianConfig() {
  return new Promise(function (resolve, reject) {
    return (0, _child_process.execFile)('/usr/bin/dpkg', ['-S', '/usr/bin/dpkg'], function (err) {
      return err ? reject(err) : resolve(distroConfig[DISTRO_DEBIAN]);
    });
  });
};

var getRPMConfig = function getRPMConfig() {
  return new Promise(function (resolve, reject) {
    return (0, _child_process.execFile)('/usr/bin/rpm', ['-q', '-f', '/usr/bin/rpm'], function (err) {
      return err ? reject(err) : resolve(distroConfig[DISTRO_RPM]);
    });
  });
};

var getDistroConfig = function () {
  var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee() {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return getDebianConfig();

          case 3:
            return _context.abrupt('return', _context.sent);

          case 6:
            _context.prev = 6;
            _context.t0 = _context['catch'](0);

          case 8:
            _context.prev = 8;
            _context.next = 11;
            return getRPMConfig();

          case 11:
            return _context.abrupt('return', _context.sent);

          case 14:
            _context.prev = 14;
            _context.t1 = _context['catch'](8);

          case 16:
            return _context.abrupt('return', false);

          case 17:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[0, 6], [8, 14]]);
  }));

  return function getDistroConfig() {
    return _ref.apply(this, arguments);
  };
}();

var getCurrentVersion = function getCurrentVersion() {
  return new Promise(function (resolve, reject) {
    (0, _child_process.execFile)(bin, ['--version'], function (err, stdout, stderr) {
      if (err || stderr) {
        resolve(null);
      } else {
        var version = stdout.match(/^Atom\s+?:\s+(\d+\.\d+\.\d+(?:-\w+\d+)?)$/im);

        resolve(version[1]);
      }
    });
  });
};

var getLatestVersion = function getLatestVersion() {
  return new Promise(function (resolve, reject) {
    return (0, _request2.default)({
      url: RELEASES_API,
      headers: { 'User-Agent': USER_AGENT },
      json: true
    }, function (err, res, body) {
      if (err) {
        reject(err);
      } else {
        var release = body.find(function (release) {
          return release.prerelease === !!_commander2.default.beta;
        });

        resolve(release.name);
      }
    });
  });
};

var getChangelog = function getChangelog(version) {
  return new Promise(function (resolve, reject) {
    return (0, _request2.default)({
      url: 'https://git.io',
      method: 'post',
      form: { url: `${RELEASE_URL}/v${version}` },
      headers: { 'User-Agent': USER_AGENT }
    }, function (err, res) {
      return err ? reject(err) : resolve(res.headers['location']);
    });
  });
};

var download = function download(version, distro) {
  return new Promise(function (resolve, reject) {
    var green = _chalk2.default.green,
        dim = _chalk2.default.dim;
    var file = distro.file;


    var opts = {
      url: `${DOWNLOAD_URL}/v${version}/${file}`,
      headers: { 'User-Agent': USER_AGENT }
    };

    try {
      _fs2.default.accessSync(TEMP_FOLDER, _fs2.default.W_OK);
    } catch (err) {
      if (err.code === 'ENOENT') {
        _fs2.default.mkdirSync(TEMP_FOLDER);
      } else {
        throw err;
      }
    }

    var dest = _path2.default.join(TEMP_FOLDER, `${version}-${file}`);
    var req = (0, _request2.default)(opts);

    req.on('response', function (res) {
      if (isTTY) {
        var len = parseInt(res.headers['content-length'], 10);
        var tmpl = `  ${file} [${green(':bar')}] :percent ${dim(':etas')}`;
        var bar = new _progress2.default(tmpl, {
          complete: '=',
          incomplete: ' ',
          width: 40,
          total: len
        });

        res.on('data', function (chunk) {
          return bar.tick(chunk.length);
        });
      }

      res.on('end', function () {
        return resolve(dest);
      });
    });

    req.on('error', function (err) {
      return reject(err);
    });

    req.pipe(_fs2.default.createWriteStream(dest));
  });
};

var install = function install(file, distro) {
  return new Promise(function (resolve, reject) {
    var cmd = distro.cmd;


    var dpkg = (0, _child_process.spawn)('sudo', [].concat(cmd, [file]), {
      stdio: ['ignore', 'inherit', 'inherit']
    });

    dpkg.on('error', function (err) {
      return reject(err);
    });
    dpkg.on('exit', function (res) {
      return resolve(res);
    });
  });
};

var acu = function () {
  var _ref2 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2() {
    var bold, underline, dim, releaseType, spinner, distro, current, latest, changelog, answer, file, result;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            bold = _chalk2.default.bold, underline = _chalk2.default.underline, dim = _chalk2.default.dim;
            releaseType = _commander2.default.beta ? 'beta' : 'stable';
            spinner = (0, _ora2.default)();


            spinner.info(`atom-check-updates v${pkg.version}`);

            spinner.text = 'Checking distro';
            spinner.start();

            _context2.prev = 6;
            _context2.next = 9;
            return getDistroConfig();

          case 9:
            distro = _context2.sent;


            if (!distro) {
              spinner.fail('You don\'t seem to be running a supported distro');

              process.exit(1);
            }

            spinner.succeed(`Detected as running on a ${bold(distro.name)}-based distro.`);

            spinner.text = `Checking for installed ${bold(releaseType)} version`;
            spinner.start();

            _context2.next = 16;
            return getCurrentVersion();

          case 16:
            current = _context2.sent;


            spinner.succeed(`Current ${bold(releaseType)} version: ` + (current ? bold(current) : 'None, or older than 1.7.0'));

            spinner.text = `Checking for latest ${bold(releaseType)} release`;
            spinner.start();

            _context2.next = 22;
            return getLatestVersion();

          case 22:
            latest = _context2.sent;
            _context2.next = 25;
            return getChangelog(latest);

          case 25:
            changelog = _context2.sent;


            if (current === latest) {
              spinner.succeed(`You're already on the latest ${bold(releaseType)} version! ` + `(changelog: ${underline(changelog)})`);

              process.exit(0);
            } else {
              spinner.info(`Latest ${bold(releaseType)} version: ${bold(latest)} ` + `(changelog: ${underline(changelog)})`);
            }

            if (_commander2.default.forceYes) {
              _context2.next = 37;
              break;
            }

            if (!isTTY) {
              _context2.next = 35;
              break;
            }

            _context2.next = 31;
            return _inquirer2.default.prompt([{
              type: 'confirm',
              name: 'confirm',
              message: `Upgrade to v${latest}?`,
              default: false
            }]);

          case 31:
            answer = _context2.sent;


            if (!answer.confirm) {
              spinner.fail('Installation aborted!');

              process.exit(1);
            }
            _context2.next = 37;
            break;

          case 35:
            spinner.fail(`Not in a TTY. Pass ${dim('--force-yes')} to continue`);

            process.exit(1);

          case 37:
            _context2.next = 39;
            return download(latest, distro);

          case 39:
            file = _context2.sent;
            _context2.next = 42;
            return install(file, distro);

          case 42:
            result = _context2.sent;


            if (result === 0) {
              spinner.succeed('Installation completed successfully!');

              _fs2.default.unlinkSync(file);

              process.exit(0);
            }
            _context2.next = 51;
            break;

          case 46:
            _context2.prev = 46;
            _context2.t0 = _context2['catch'](6);

            spinner.fail('An unexpected error occured');
            process.stderr.write(_context2.t0.toString());

            process.exit(1);

          case 51:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined, [[6, 46]]);
  }));

  return function acu() {
    return _ref2.apply(this, arguments);
  };
}();

exports.default = acu;